type ShakuDirectiveUnderline = {
    type: "DirectiveUnderline";
    config: {
        style: "solid" | "dotted" | "wavy";
        content: string;
        offset: number;
        isEscaped: boolean;
    };
};
type ShakuAnnotationLine = {
    type: "AnnotationLine";
    config: {
        offset: number;
        content: string;
        isEscaped: boolean;
    };
};
type ShakuDirectiveHighlightInline = {
    type: "DirectiveHighlightInline";
    config: {
        isEscaped: boolean;
        parts: Array<{
            offset: number;
            length: number;
            id?: number;
        }>;
    };
};
type ShakuDirectiveCallout = {
    type: "DirectiveCallout";
    config: {
        isEscaped: boolean;
        offset: number;
    };
};
type ShakuDirectiveCollapse = {
    type: "DirectiveCollapse";
    config: {
        isEscaped: boolean;
        offset: number;
        mark: "start" | "end";
    };
};
type ShakuDirectiveHighlight = {
    type: "DirectiveHighlight";
    config: {
        isEscaped: boolean;
        mark: "start" | "end" | "below";
    };
};
type ShakuDirectiveDim = {
    type: "DirectiveDim";
    config: {
        isEscaped: boolean;
        mark: "start" | "end" | "below";
    };
};
type ShakuDirectiveFocus = {
    type: "DirectiveFocus";
    config: {
        isEscaped: boolean;
        mark: "start" | "end" | "below";
    };
};
type ShakuDirectiveDiff = {
    type: "DirectiveDiff";
    config: {
        isEscaped: boolean;
        type: "+" | "-";
        mark: "start" | "end" | "below";
    };
};
type ShakuLine = ShakuDirectiveUnderline | ShakuAnnotationLine | ShakuDirectiveCallout | ShakuDirectiveCollapse | ShakuDirectiveHighlight | ShakuDirectiveDim | ShakuDirectiveFocus | ShakuDirectiveHighlightInline | ShakuDirectiveDiff;
declare const parseLine: (line: string) => ShakuLine | null;
declare function shouldApplyAnnotation(meta: string): boolean;
/**
 * Thought we process by lines, but we need to render in blocks, for instance:
 * A full callout normally contains:
 * 1. DirectiveCallout
 * 2. one or more AnnotationLine
 *
 * An underline annotation contains:
 * 1. DirectiveUnderline
 * 2. one ore more AnnotationLine
 *
 * DirectiveCollapse also render a few lines of source code
 * So this function should render component
 */
type ShakuComponentCallout = {
    type: "ShakuComponentCallout";
    config: {
        offset: number;
        arrowOffset: number;
        /** restricted raw html */
        contents: string;
    };
};
type ShakuComponentUnderline = {
    type: "ShakuComponentUnderline";
    config: {
        offset: number;
        underlineOffset: number;
        underlineContent: string;
        underlineStyle: "solid" | "dotted" | "wavy";
        /** restricted raw html */
        contents: string;
    };
};
type ShakuComponent = ShakuComponentCallout | ShakuComponentUnderline;
declare function renderComponent(component: ShakuComponent, options?: {
    useDangerousRawHtml?: boolean;
}): string | undefined;

export { ShakuAnnotationLine, ShakuDirectiveCallout, ShakuDirectiveCollapse, ShakuDirectiveDiff, ShakuDirectiveDim, ShakuDirectiveFocus, ShakuDirectiveHighlight, ShakuDirectiveHighlightInline, ShakuDirectiveUnderline, ShakuLine, parseLine, renderComponent, shouldApplyAnnotation };
